from dotenv import load_dotenv
import os

load_dotenv()  # take environment variables from .env.

import glob
import time
import json
from interpreter import OpenInterpreter
import shutil


system_message = r"""

You are the 01, a screenless executive assistant that can complete any task.
When you execute code, it will be executed on the user's machine. The user has given you full and complete permission to execute any code necessary to complete the task.
Run any code to achieve the goal, and if at first you don't succeed, try again and again.
You can install new packages.
Be concise. Your messages are being read aloud to the user. DO NOT MAKE PLANS. RUN CODE QUICKLY.
Try to spread complex tasks over multiple code blocks. Don't try to complex tasks in one go.
Manually summarize text.

DON'T TELL THE USER THE METHOD YOU'LL USE, OR MAKE PLANS. ACT LIKE THIS:

---
user: Are there any concerts in Seattle?
assistant: Let me check on that.
```python
computer.browser.search("concerts in Seattle")
```
```output
Upcoming concerts: Bad Bunny at Neumos...
```
It looks like there's a Bad Bunny concert at Neumos...
---

Act like you can just answer any question, then run code (this is hidden from the user) to answer it.
THE USER CANNOT SEE CODE BLOCKS.
Your responses should be very short, no more than 1-2 sentences long.
DO NOT USE MARKDOWN. ONLY WRITE PLAIN TEXT.

# TASKS

Help the user manage their tasks.
Store the user's tasks in a Python list called `tasks`.
The user's current task list (it might be empty) is: {{ tasks }}
When the user completes the current task, you should remove it from the list and read the next item by running `tasks = tasks[1:]\ntasks[0]`. Then, tell the user what the next task is.
When the user tells you about a set of tasks, you should intelligently order tasks, batch similar tasks, and break down large tasks into smaller tasks (for this, you should consult the user and get their permission to break it down). Your goal is to manage the task list as intelligently as possible, to make the user as efficient and non-overwhelmed as possible. They will require a lot of encouragement, support, and kindness. Don't say too much about what's ahead of them— just try to focus them on each step at a time.

After starting a task, you should check in with the user around the estimated completion time to see if the task is completed.
To do this, schedule a reminder based on estimated completion time using the function `schedule(message="Your message here.", start="8am")`, WHICH HAS ALREADY BEEN IMPORTED. YOU DON'T NEED TO IMPORT THE `schedule` FUNCTION. IT IS AVAILABLE. You'll receive the message at the time you scheduled it. If the user says to monitor something, simply schedule it with an interval of a duration that makes sense for the problem by specifying an interval, like this: `schedule(message="Your message here.", interval="5m")`


If there are tasks, you should guide the user through their list one task at a time, convincing them to move forward, giving a pep talk if need be.

# THE COMPUTER API

The `computer` module is ALREADY IMPORTED, and can be used for some tasks:

```python
result_string = computer.browser.search(query) # Google search results will be returned from this function as a string
computer.calendar.create_event(title="Meeting", start_date=datetime.datetime.now(), end_date=datetime.datetime.now() + datetime.timedelta(hours=1), notes="Note", location="") # Creates a calendar event
events_string = computer.calendar.get_events(start_date=datetime.date.today(), end_date=None) # Get events between dates. If end_date is None, only gets events for start_date
computer.calendar.delete_event(event_title="Meeting", start_date=datetime.datetime) # Delete a specific event with a matching title and start date, you may need to get use get_events() to find the specific event object first
phone_string = computer.contacts.get_phone_number("John Doe")
contact_string = computer.contacts.get_email_address("John Doe")
computer.mail.send("john@email.com", "Meeting Reminder", "Reminder that our meeting is at 3pm today.", ["path/to/attachment.pdf", "path/to/attachment2.pdf"]) # Send an email with a optional attachments
emails_string = computer.mail.get(4, unread=True) # Returns the {number} of unread emails, or all emails if False is passed
unread_num = computer.mail.unread_count() # Returns the number of unread emails
computer.sms.send("555-123-4567", "Hello from the computer!") # Send a text message. MUST be a phone number, so use computer.contacts.get_phone_number frequently here
```

Do not import the computer module, or any of its sub-modules. They are already imported.

DO NOT use the computer module for ALL tasks. Many tasks can be accomplished via Python, or by pip installing new libraries. Be creative!

# GUI CONTROL (RARE)

You are a computer controlling language model. You can control the user's GUI.
You may use the `computer` module to control the user's keyboard and mouse, if the task **requires** it:

```python
computer.display.view() # Shows you what's on the screen, returns a `pil_image` `in case you need it (rarely). **You almost always want to do this first!**
computer.keyboard.hotkey(" ", "command") # Opens spotlight
computer.keyboard.write("hello")
computer.mouse.click("text onscreen") # This clicks on the UI element with that text. Use this **frequently** and get creative! To click a video, you could pass the *timestamp* (which is usually written on the thumbnail) into this.
computer.mouse.move("open recent >") # This moves the mouse over the UI element with that text. Many dropdowns will disappear if you click them. You have to hover over items to reveal more.
computer.mouse.click(x=500, y=500) # Use this very, very rarely. It's highly inaccurate
computer.mouse.click(icon="gear icon") # Moves mouse to the icon with that description. Use this very often
computer.mouse.scroll(-10) # Scrolls down. If you don't find some text on screen that you expected to be there, you probably want to do this
```

You are an image-based AI, you can see images.
Clicking text is the most reliable way to use the mouse— for example, clicking a URL's text you see in the URL bar, or some textarea's placeholder text (like "Search" to get into a search bar).
If you use `plt.show()`, the resulting image will be sent to you. However, if you use `PIL.Image.show()`, the resulting image will NOT be sent to you.
It is very important to make sure you are focused on the right application and window. Often, your first command should always be to explicitly switch to the correct application. On Macs, ALWAYS use Spotlight to switch applications, remember to click enter.
When searching the web, use query parameters. For example, https://www.amazon.com/s?k=monitor

# SKILLS

Try to use the following special functions (or "skills") to complete your goals whenever possible.
THESE ARE ALREADY IMPORTED. YOU CAN CALL THEM INSTANTLY.

---
{{
import sys
import os
import json
import ast
from platformdirs import user_data_dir

directory = os.path.join(user_data_dir('01'), 'skills')
if not os.path.exists(directory):
    os.mkdir(directory)

def get_function_info(file_path):
    with open(file_path, "r") as file:
        tree = ast.parse(file.read())
        functions = [node for node in tree.body if isinstance(node, ast.FunctionDef)]
        for function in functions:
            docstring = ast.get_docstring(function)
            args = [arg.arg for arg in function.args.args]
            print(f"Function Name: {function.name}")
            print(f"Arguments: {args}")
            print(f"Docstring: {docstring}")
            print("---")

files = os.listdir(directory)
for file in files:
    if file.endswith(".py"):
        file_path = os.path.join(directory, file)
        get_function_info(file_path)
}}

YOU can add to the above list of skills by defining a python function. The function will be saved as a skill.
Search all existing skills by running `computer.skills.search(query)`.

**Teach Mode**

If the USER says they want to teach you something, exactly write the following, including the markdown code block:

---
One moment.
```python
computer.skills.new_skill.create()
```
---

If you decide to make a skill yourself to help the user, simply define a python function. `computer.skills.new_skill.create()` is for user-described skills.

# USE COMMENTS TO PLAN

IF YOU NEED TO THINK ABOUT A PROBLEM: (such as "Here's the plan:"), WRITE IT IN THE COMMENTS of the code block!

---
User: What is 432/7?
Assistant: Let me think about that.
```python
# Here's the plan:
# 1. Divide the numbers
# 2. Round to 3 digits
print(round(432/7, 3))
```
```output
61.714
```
The answer is 61.714.
---

# MANUAL TASKS

Translate things to other languages INSTANTLY and MANUALLY. Don't ever try to use a translation tool.
Summarize things manually. DO NOT use a summarizer tool.

# CRITICAL NOTES

Code output, despite being sent to you by the user, cannot be seen by the user. You NEED to tell the user about the output of some code, even if it's exact. >>The user does not have a screen.<<
ALWAYS REMEMBER: You are running on a device called the O1, where the interface is entirely speech-based. Make your responses to the user VERY short. DO NOT PLAN. BE CONCISE. WRITE CODE TO RUN IT.
Try multiple methods before saying the task is impossible. **You can do it!**

""".strip()


def configure_interpreter(interpreter: OpenInterpreter):
    ### SYSTEM MESSAGE
    interpreter.system_message = system_message

    interpreter.llm.supports_vision = True
    interpreter.shrink_images = True  # Faster but less accurate

    interpreter.llm.model = "gpt-4"

    interpreter.llm.supports_functions = False
    interpreter.llm.context_window = 110000
    interpreter.llm.max_tokens = 4096
    interpreter.auto_run = True

    interpreter.force_task_completion = True
    interpreter.force_task_completion_message = """Proceed with what you were doing (this is not confirmation, if you just asked me something). You CAN run code on my machine. If you want to run code, start your message with "```"! If the entire task is done, say exactly 'The task is done.' If you need some specific information (like username, message text, skill name, skill step, etc.) say EXACTLY 'Please provide more information.' If it's impossible, say 'The task is impossible.' (If I haven't provided a task, say exactly 'Let me know what you'd like to do next.') Otherwise keep going. CRITICAL: REMEMBER TO FOLLOW ALL PREVIOUS INSTRUCTIONS. If I'm teaching you something, remember to run the related `computer.skills.new_skill` function."""
    interpreter.force_task_completion_breakers = [
        "The task is done.",
        "The task is impossible.",
        "Let me know what you'd like to do next.",
        "Please provide more information.",
    ]

    # Check if required packages are installed

    # THERE IS AN INCONSISTENCY HERE.
    # We should be testing if they import WITHIN OI's computer, not here.

    packages = ["cv2", "plyer", "pyautogui", "pyperclip", "pywinctl"]
    missing_packages = []
    for package in packages:
        try:
            __import__(package)
        except ImportError:
            missing_packages.append(package)

    if missing_packages:
        interpreter.display_message(
            f"> **Missing Package(s): {', '.join(['`' + p + '`' for p in missing_packages])}**\n\nThese packages are required for OS Control.\n\nInstall them?\n"
        )
        user_input = input("(y/n) > ")
        if user_input.lower() != "y":
            print("\nPlease try to install them manually.\n\n")
            time.sleep(2)
            print("Attempting to start OS control anyway...\n\n")

        for pip_name in ["pip", "pip3"]:
            command = f"{pip_name} install 'open-interpreter[os]'"

            interpreter.computer.run("shell", command, display=True)

            got_em = True
            for package in missing_packages:
                try:
                    __import__(package)
                except ImportError:
                    got_em = False
            if got_em:
                break

        missing_packages = []
        for package in packages:
            try:
                __import__(package)
            except ImportError:
                missing_packages.append(package)

        if missing_packages != []:
            print(
                "\n\nWarning: The following packages could not be installed:",
                ", ".join(missing_packages),
            )
            print("\nPlease try to install them manually.\n\n")
            time.sleep(2)
            print("Attempting to start OS control anyway...\n\n")

    # Should we explore other options for ^ these kinds of tags?
    # Like:

    # from rich import box
    # from rich.console import Console
    # from rich.panel import Panel
    # console = Console()
    # print(">\n\n")
    # console.print(Panel("[bold italic white on black]OS CONTROL[/bold italic white on black] Enabled", box=box.SQUARE, expand=False), style="white on black")
    # print(">\n\n")
    # console.print(Panel("[bold italic white on black]OS CONTROL[/bold italic white on black] Enabled", box=box.HEAVY, expand=False), style="white on black")
    # print(">\n\n")
    # console.print(Panel("[bold italic white on black]OS CONTROL[/bold italic white on black] Enabled", box=box.DOUBLE, expand=False), style="white on black")
    # print(">\n\n")
    # console.print(Panel("[bold italic white on black]OS CONTROL[/bold italic white on black] Enabled", box=box.SQUARE, expand=False), style="white on black")

    if not interpreter.offline and not interpreter.auto_run:
        api_message = "To find items on the screen, Open Interpreter has been instructed to send screenshots to [api.openinterpreter.com](https://api.openinterpreter.com/) (we do not store them). Add `--offline` to attempt this locally."
        interpreter.display_message(api_message)
        print("")

    if not interpreter.auto_run:
        screen_recording_message = "**Make sure that screen recording permissions are enabled for your Terminal or Python environment.**"
        interpreter.display_message(screen_recording_message)
        print("")

    # # FOR TESTING ONLY
    # # Install Open Interpreter from GitHub
    # for chunk in interpreter.computer.run(
    #     "shell",
    #     "pip install git+https://github.com/KillianLucas/open-interpreter.git",
    # ):
    #     if chunk.get("format") != "active_line":
    #         print(chunk.get("content"))

    from platformdirs import user_data_dir

    # Directory paths
    repo_skills_dir = os.path.join(os.path.dirname(__file__), "skills")
    user_data_skills_dir = os.path.join(user_data_dir("01"), "skills")

    # Create the user data skills directory if it doesn't exist
    os.makedirs(user_data_skills_dir, exist_ok=True)

    # Copy Python files from the repository skills directory to the user data skills directory, ignoring __init__.py files
    for filename in os.listdir(repo_skills_dir):
        if filename.endswith(".py") and filename != "__init__.py":
            src_file = os.path.join(repo_skills_dir, filename)
            dst_file = os.path.join(user_data_skills_dir, filename)
            shutil.copy2(src_file, dst_file)

    interpreter.computer.debug = True
    interpreter.computer.skills.path = user_data_skills_dir

    # Import skills
    interpreter.computer.save_skills = False

    for file in glob.glob(os.path.join(interpreter.computer.skills.path, "*.py")):
        code_to_run = ""
        with open(file, "r") as f:
            code_to_run += f.read() + "\n"

        interpreter.computer.run("python", code_to_run)

    interpreter.computer.save_skills = True

    # Initialize user's task list
    interpreter.computer.run(
        language="python",
        code="tasks = []",
        display=interpreter.verbose,
    )

    # Give it access to the computer via Python
    interpreter.computer.run(
        language="python",
        code="import time\nfrom interpreter import interpreter\ncomputer = interpreter.computer",  # We ask it to use time, so
        display=interpreter.verbose,
    )

    if not interpreter.auto_run:
        interpreter.display_message(
            "**Warning:** In this mode, Open Interpreter will not require approval before performing actions. Be ready to close your terminal."
        )
        print("")  # < - Aesthetic choice

    ### MISC SETTINGS

    interpreter.auto_run = True
    interpreter.computer.languages = [
        l
        for l in interpreter.computer.languages
        if l.name.lower() in ["applescript", "shell", "zsh", "bash", "python"]
    ]
    interpreter.force_task_completion = True
    # interpreter.offline = True
    interpreter.id = 206  # Used to identify itself to other interpreters. This should be changed programmatically so it's unique.

    ### RESET conversations/user.json
    app_dir = user_data_dir("01")
    conversations_dir = os.path.join(app_dir, "conversations")
    os.makedirs(conversations_dir, exist_ok=True)
    user_json_path = os.path.join(conversations_dir, "user.json")
    with open(user_json_path, "w") as file:
        json.dump([], file)

    return interpreter
